@@ -93,7 +93,7 @@ class Iteralign(AbstractCommand):
         -m, --minimap2           If set, use minimap2 instead of bowtie2 for
                                  the alignment.
         -l, --min_len=INT        Length to which the reads should be
-                                 truncated [default: 40].
+                                 truncated [default: 20].
         -o, --out_sam=FILE       Path where the alignment will be written in
                                  SAM format.
     """
@@ -181,9 +181,7 @@ class Digest(AbstractCommand):
         )
 
         hcd.frag_len(
-            output_dir=self.args["--outdir"],
-            plot=self.args["--plot"],
-            fig_path=figpath,
+            output_dir=self.args["--outdir"], plot=self.args["--plot"], fig_path=figpath
         )
 
 
@@ -343,6 +341,7 @@ class View(AbstractCommand):
         if self.args["--log"]:
             binned_map = binned_map.log1p()
 
+        self.vmax = np.percentile(binned_map.data, self.vmax)
         # ZOOM REGION
         if self.args["--region"]:
             if not self.args["--frags"]:
@@ -353,9 +352,7 @@ class View(AbstractCommand):
                 )
                 sys.exit(1)
             # Load positions from fragments list
-            reg_pos = pd.read_csv(
-                self.args["--frags"], delimiter="\t", usecols=(1, 2)
-            )
+            reg_pos = pd.read_csv(self.args["--frags"], delimiter="\t", usecols=(1, 2))
             # Readjust bin coords post binning
             if self.binning:
                 if self.bp_unit:
@@ -365,9 +362,7 @@ class View(AbstractCommand):
                     num_binned = binned_start[1:] - binned_start[:-1]
                     chr_names = np.unique(reg_pos.iloc[:, 0])
                     binned_chrom = np.repeat(chr_names, num_binned)
-                    reg_pos = pd.DataFrame(
-                        {0: binned_chrom, 1: binned_frags[:, 0]}
-                    )
+                    reg_pos = pd.DataFrame({0: binned_chrom, 1: binned_frags[:, 0]})
                 else:
                     reg_pos = reg_pos.iloc[:: self.binning, :]
 
@@ -391,8 +386,7 @@ class View(AbstractCommand):
                 trim_std = float(self.args["--trim"])
             except ValueError:
                 print(
-                    "You must specify a number of standard deviations for "
-                    "trimming"
+                    "You must specify a number of standard deviations for " "trimming"
                 )
                 raise
             binned_map = hcs.trim_sparse(binned_map, n_std=trim_std)
@@ -403,6 +397,7 @@ class View(AbstractCommand):
 
         input_map = self.args["<contact_map>"]
         cmap = self.args["--cmap"]
+        self.vmax = float(self.args["--max"])
         self.bp_unit = False
         bin_str = self.args["--binning"].upper()
         try:
@@ -428,7 +423,6 @@ class View(AbstractCommand):
                 )
                 raise
 
-        vmax = float(self.args["--max"])
         output_file = self.args["--output"]
         sparse_map = hio.load_sparse_matrix(input_map)
         processed_map = self.process_matrix(sparse_map)
@@ -455,17 +449,16 @@ class View(AbstractCommand):
 
         if self.args["--despeckle"]:
             processed_map = hcs.despeckle_simple(processed_map)
-        vmax = np.percentile(processed_map.data, vmax)
         try:
             dense_map = hcv.sparse_to_dense(processed_map, remove_diag=False)
-            vmin = 0
+            self.vmin = 0
             if self.args["<contact_map2>"]:
-                vmin, vmax = -2, 2
+                self.vmin, self.vmax = -2, 2
             hcv.plot_matrix(
                 dense_map,
                 filename=output_file,
-                vmin=vmin,
-                vmax=vmax,
+                vmin=self.vmin,
+                vmax=self.vmax,
                 cmap=cmap,
             )
         except MemoryError:
@@ -792,16 +785,12 @@ class Rebin(AbstractCommand):
             hic_map, _ = hcs.bin_bp_sparse(hic_map, frags.start_pos, binning)
             for chrom in chromnames:
                 # For all chromosomes, get new bin start positions
-                bin_id = (
-                    frags.loc[frags.chrom == chrom, "start_pos"] // binning
-                )
+                bin_id = frags.loc[frags.chrom == chrom, "start_pos"] // binning
                 frags.loc[frags.chrom == chrom, "id"] = bin_id + 1
                 frags.loc[frags.chrom == chrom, "start_pos"] = binning * bin_id
                 bin_ends = binning * bin_id + binning
                 # Do not allow bin ends to be larger than chrom size
-                chromsize = chromlist.length[chromlist.contig == chrom].values[
-                    0
-                ]
+                chromsize = chromlist.length[chromlist.contig == chrom].values[0]
                 # bin_ends.iloc[-1] = min([bin_ends.iloc[-1], chromsize])
                 bin_ends[bin_ends > chromsize] = chromsize
                 frags.loc[frags.chrom == chrom, "end_pos"] = bin_ends
@@ -831,9 +820,7 @@ class Rebin(AbstractCommand):
         for chrom in chromnames:
             n_bins = frags.start_pos[frags.chrom == chrom].shape[0]
             chromlist.loc[chromlist.contig == chrom, "n_frags"] = n_bins
-            chromlist.loc[
-                chromlist.contig == chrom, "cumul_length"
-            ] = cumul_bins
+            chromlist.loc[chromlist.contig == chrom, "cumul_length"] = cumul_bins
             cumul_bins += n_bins
 
         # Write 3 binned output files
@@ -922,13 +909,9 @@ class Convert(AbstractCommand):
             print("not implemented yet, sorry")
         elif out_fmt == "GRAAL":
             mat_name = (
-                prefix + ".mat.tsv"
-                if prefix
-                else "abs_fragments_contacts_weighted.txt"
-            )
-            frag_name = (
-                prefix + ".frag.tsv" if prefix else "fragments_list.txt"
+                prefix + ".mat.tsv" if prefix else "abs_fragments_contacts_weighted.txt"
             )
+            frag_name = prefix + ".frag.tsv" if prefix else "fragments_list.txt"
             chr_name = prefix + ".chr.tsv" if prefix else "info_contigs.txt"
             out_mat = join(out_path, mat_name)
             out_frag = join(out_path, frag_name)
@@ -972,12 +955,15 @@ def parse_bin_str(bin_str):
     binning : int
         The number of basepair corresponding to the binning string.
     """
-    bin_str = bin_str.upper()
-    binsuffix = {"B": 1, "K": 1000, "M": 1e6, "G": 1e9}
-    unit_pos = re.search(r"[KMG]?B[P]?$", bin_str).start()
-    bp_unit = bin_str[unit_pos:]
-    # Extract unit and multiply accordingly for fixed bp binning
-    binning = int(float(bin_str[:unit_pos]) * binsuffix[bp_unit[0]])
+    try:
+        binning = int(bin_str)
+    except ValueError:
+        bin_str = bin_str.upper()
+        binsuffix = {"B": 1, "K": 1000, "M": 1e6, "G": 1e9}
+        unit_pos = re.search(r"[KMG]?B[P]?$", bin_str).start()
+        bp_unit = bin_str[unit_pos:]
+        # Extract unit and multiply accordingly for fixed bp binning
+        binning = int(float(bin_str[:unit_pos]) * binsuffix[bp_unit[0]])
 
     return binning
 
@@ -1000,18 +986,27 @@ def parse_ucsc(ucsc_str, bins):
     coord : tuple
         A tuple containing the bin range containing in the requested region.
     """
-    chrom, bp = ucsc_str.split(":")
-    bp = bp.replace(",", "").upper()
-    start, end = bp.split("-")
-    try:
-        start, end = int(start), int(end)
-    except ValueError:
+    if ":" in ucsc_str:
+        chrom, bp = ucsc_str.split(":")
+        bp = bp.replace(",", "").upper()
+        start, end = bp.split("-")
         start, end = parse_bin_str(start), parse_bin_str(end)
-    # Make absolute bin index (independent of chrom)
-    bins["id"] = bins.index
-    chrombins = bins.loc[bins.iloc[:, 0] == chrom, :]
-    start = max([start, 1])
-    start = max(chrombins.id[chrombins.iloc[:, 1] // start == 0])
-    end = max(chrombins.id[chrombins.iloc[:, 1] // end == 0])
-    coord = (start, end)
+        # Make absolute bin index (independent of chrom)
+        bins["id"] = bins.index
+        chrombins = bins.loc[bins.iloc[:, 0] == chrom, :]
+        start = max([start, 1])
+        start = max(chrombins.id[chrombins.iloc[:, 1] // start == 0])
+        end = max(chrombins.id[chrombins.iloc[:, 1] // end == 0])
+    else:
+        chrom = ucsc_str
+        # Make absolute bin index (independent of chrom)
+        bins["id"] = bins.index
+        chrombins = bins.loc[bins.iloc[:, 0] == chrom, :]
+        try:
+            start = min(chrombins.id)
+            end = max(chrombins.id)
+        except ValueError:
+            print("Invalid chromosome")
+            raise
+    coord = (int(start), int(end))
     return coord
