@@ -34,6 +34,7 @@ import random
 import multiprocessing as mp
 import pandas as pd
 import sys
+from hicstuff.log import logger
 
 
 def despeckle_simple(B, th2=2, threads=1):
@@ -103,7 +104,9 @@ def despeckle_local(M, stds=2, width=2):
 
     N = np.array(M, dtype=np.float64)
     n, m = M.shape
-    for i, j in itertools.product(range(width, n - width), range(width, m - width)):
+    for i, j in itertools.product(
+        range(width, n - width), range(width, m - width)
+    ):
         square = M[i - width : i + width, j - width : j + width]
         avg = np.average(square)
         std = np.std(square)
@@ -145,7 +148,9 @@ def bin_dense(M, subsampling_factor=3):
         ).sum(axis=(0, 2))
         C = (
             remaining_col.T.reshape(
-                m % subsampling_factor, m // subsampling_factor, subsampling_factor
+                m % subsampling_factor,
+                m // subsampling_factor,
+                subsampling_factor,
             )
             .sum(axis=(0, 2))
             .T
@@ -208,7 +213,9 @@ def bin_annotation(annotation=None, subsampling_factor=3):
     if annotation is None:
         annotation = np.array([])
     n = len(annotation)
-    binned_positions = [annotation[i] for i in range(n) if i % subsampling_factor == 0]
+    binned_positions = [
+        annotation[i] for i in range(n) if i % subsampling_factor == 0
+    ]
     if len(binned_positions) == 0:
         binned_positions.append(0)
     return np.array(binned_positions)
@@ -224,7 +231,9 @@ def bin_measurement(measurement=None, subsampling_factor=3):
         measurement = np.array([])
     n = len(measurement)
     binned_measurement = [
-        measurement[i - subs + 1 : i].sum() for i in range(n) if i % subs == 0 and i > 0
+        measurement[i - subs + 1 : i].sum()
+        for i in range(n)
+        if i % subs == 0 and i > 0
     ]
     return np.array(binned_measurement)
 
@@ -236,7 +245,9 @@ def build_pyramid(M, subsampling_factor=3):
 
     subs = int(subsampling_factor)
     if subs < 1:
-        raise ValueError("Subsampling factor needs to be an integer greater than 1.")
+        raise ValueError(
+            "Subsampling factor needs to be an integer greater than 1."
+        )
     N = [M]
     while min(N[-1].shape) > 1:
         N.append(bin_matrix(N[-1], subsampling_factor=subs))
@@ -288,7 +299,9 @@ def bin_bp_dense(M, positions, bin_len=10000):
     # Use (chr, bin#) as grouping key (coord) and indices of fragments
     # belonging to current bin (bin_frags)
     bin_No = 0
-    for coords, bin_frags in itertools.groupby(frag_idx, lambda x: tuple(frags[x, :])):
+    for coords, bin_frags in itertools.groupby(
+        frag_idx, lambda x: tuple(frags[x, :])
+    ):
         bin_frags = list(bin_frags)
         first_frag, last_frag = bin_frags[0], bin_frags[-1] + 1
         out_pos[bin_No] = coords[1] * bin_len
@@ -321,7 +334,9 @@ def bin_exact_bp_dense(M, positions, bin_len=10000):
     """
     units = positions / bin_len
     n = len(positions)
-    idx = [i for i in range(n - 1) if np.ceil(units[i]) < np.ceil(units[i + 1])]
+    idx = [
+        i for i in range(n - 1) if np.ceil(units[i]) < np.ceil(units[i + 1])
+    ]
     m = len(idx) - 1
     N = np.zeros((m, m))
     remainders = [0] + [np.abs(units[i] - units[i + 1]) for i in range(m)]
@@ -379,7 +394,9 @@ def bin_bp_sparse(M, positions, bin_len=10000):
     bin_No = 0
     # Use (chr, bin) as grouping key (coord) and indices of fragments
     # belonging to current bin (bin_frags)
-    for coords, bin_frags in itertools.groupby(frag_idx, lambda x: tuple(frags[x, :])):
+    for coords, bin_frags in itertools.groupby(
+        frag_idx, lambda x: tuple(frags[x, :])
+    ):
         bin_frags = list(bin_frags)
         first_frag, last_frag = bin_frags[0], bin_frags[-1] + 1
         # Pool row/col number by bin
@@ -559,7 +576,10 @@ def normalize_dense(M, norm="SCN", order=1, iterations=3):
         except ImportError as e:
             print(str(e))
             print("I can't find mirnylib.")
-            print("Please install it from " "https://bitbucket.org/mirnylab/mirnylib")
+            print(
+                "Please install it from "
+                "https://bitbucket.org/mirnylab/mirnylib"
+            )
             print("I will use default norm as fallback.")
             return normalize_dense(M, order=order, iterations=iterations)
 
@@ -588,7 +608,7 @@ def normalize_sparse(M, norm="SCN", order=1, iterations=3):
     Parameters
     ----------
     M : scipy.sparse.csr_matrix of floats
-    norm : str
+    norm : str or callable
         Normalization procedure to use. Can be one of "SCN",
         "mirnylib", "frag" or "global". Can also be a user-
         defined function.
@@ -605,7 +625,7 @@ def normalize_sparse(M, norm="SCN", order=1, iterations=3):
     """
     # Making full symmetric matrix if not symmetric already (e.g. upper triangle)
     r = csr_matrix(M)
-    if (abs(r-r.T)>1e-10).nnz != 0:
+    if (abs(r - r.T) > 1e-10).nnz != 0:
         r += r.T
         r.setdiag(r.diagonal() / 2)
         r.eliminate_zeros()
@@ -626,15 +646,16 @@ def normalize_sparse(M, norm="SCN", order=1, iterations=3):
 
             r = linalg.norm(M, ord=order)
         except (ImportError, AttributeError) as e:
-            print(str(e))
-            print("I can't import linalg tools for sparse matrices.")
-            print("Please upgrade your scipy version to 0.16.0.")
+            logger.error(
+                "I can't import linalg tools for sparse matrices."
+                "Please upgrade your scipy version to 0.16.0."
+            )
 
     elif callable(norm):
         r = norm(M)
 
     else:
-        print("Unknown norm. Returning input as fallback")
+        logger.error("Unknown norm. Returning input as fallback")
 
     return r
 
@@ -664,31 +685,30 @@ def GC_partial(portion: str):
 
 def GC_wide(genome: str, window=1000):
     """Compute GC across a window of given length.
-    :note: Requires Biopython
+
     Parameters
     ----------
     genome : str
         The genome on which GC content will be computed.
     window : int
         The window size in which GC content is measured.
-    Returns
-    -------
-    list :
-        A list of floats, each being the GC percent in a window.
+
+    Note
+    ----
+    Biopython is required.
     """
 
-    GC = []
     from Bio import SeqIO
 
     with open(genome) as handle:
-        sequence = "".join([str(record.seq) for record in SeqIO.parse(handle, "fasta")])
+        sequence = "".join(
+            [str(record.seq) for record in SeqIO.parse(handle, "fasta")]
+        )
 
     n = len(sequence)
     for i in range(0, n, window):
         portion = sequence[i : min(i + window, n)]
-        GC.append(GC_partial(portion))
-
-    return GC
+        yield GC_partial(portion)
 
 
 def split_genome(genome, chunk_size=10000):
@@ -843,7 +863,10 @@ def domainogram(M, window=None, circ=False, extrapolate=True):
     ]
 
     if circ:
-        d += [M[i:, i:].sum() + M[: n - i, n - i].sum() for i in range(n - window, n)]
+        d += [
+            M[i:, i:].sum() + M[: n - i, n - i].sum()
+            for i in range(n - window, n)
+        ]
     elif extrapolate:
         d += [
             M[i - window :, i - window :].sum()
@@ -867,7 +890,10 @@ def from_structure(structure):
             structure = p.get_structure("S", structure)
         if isinstance(structure, PDB.Structure.Structure):
             for _ in structure.get_chains():
-                atoms = [np.array(atom.get_coord()) for atom in structure.get_atoms()]
+                atoms = [
+                    np.array(atom.get_coord())
+                    for atom in structure.get_atoms()
+                ]
     except ImportError:
         print("Biopython not found.")
         raise
@@ -980,7 +1006,12 @@ def get_missing_bins(original, trimmed):
 
 
 def to_pdb(
-    structure, filename, contigs=None, annotations=None, indices=None, special_bins=None
+    structure,
+    filename,
+    contigs=None,
+    annotations=None,
+    indices=None,
+    special_bins=None,
 ):
     """From a structure (or matrix) generate the corresponding pdb file
     representing each chain as a contig/chromosome and filling the occupancy
@@ -1030,7 +1061,9 @@ def to_pdb(
             line += " "  # 17 alternate location indicator
             line += "SOL"  # 18-20 residue name
             line += " "  # 21 unused
-            line += letters[int(contigs[indices[i]] - 1)]  # 22 chain identifier
+            line += letters[
+                int(contigs[indices[i]] - 1)
+            ]  # 22 chain identifier
             line += str(i).rjust(4)  # 23-26 residue sequence number
             line += " "  # 27 code for insertion of residues
             line += "   "  # 28-30 unused
@@ -1142,7 +1175,10 @@ def distance_law(matrix, indices=None, log_bins=True, base=1.1):
     else:
         included_bins[indices] = True
     D = np.array(
-        [np.average(matrix.diagonal(j)[included_bins[: n - j]]) for j in range(n)]
+        [
+            np.average(matrix.diagonal(j)[included_bins[: n - j]])
+            for j in range(n)
+        ]
     )
     if not log_bins:
         return np.array(range(len(D))), D
@@ -1157,7 +1193,10 @@ def distance_law(matrix, indices=None, log_bins=True, base=1.1):
             print("Not enough bins. Increase logarithm base.")
             return np.array(range(len(D))), D
         logD = np.array(
-            [np.average(D[logbin[i - 1] : logbin[i]]) for i in range(1, len(logbin))]
+            [
+                np.average(D[logbin[i - 1] : logbin[i]])
+                for i in range(1, len(logbin))
+            ]
         )
         return logbin[:-1], logD
 
@@ -1214,7 +1253,9 @@ def distance_law_multi(
     if centro_pos is not None:
         # Sanity check: as many chroms as centromeres
         if len(chr_bins) != len(centro_pos):
-            sys.stderr.write("ERROR: Number of chromosomes and centromeres differ.")
+            sys.stderr.write(
+                "ERROR: Number of chromosomes and centromeres differ."
+            )
             sys.exit(1)
         # Get bins of centromeres
         centro_bins = np.zeros(len(centro_pos))
@@ -1245,7 +1286,9 @@ def distance_law_multi(
         else:
             chr_start, chr_end = int(chr_bins[chrm]), matrix.shape[0]
         # Subset indices and matrices. Shift indices to start in chrom
-        chr_idx_mask = np.where((indices >= chr_start) & (indices < chr_end))[0]
+        chr_idx_mask = np.where((indices >= chr_start) & (indices < chr_end))[
+            0
+        ]
         usebins = np.array(indices[chr_idx_mask]) - chr_start
         chr_mat = matrix[chr_start:chr_end, chr_start:chr_end]
         # Get the ps for the different arms/chroms
@@ -1289,8 +1332,8 @@ def average_ps(ps):
 
 
 def subsample_contacts(M, prop):
-    """
-    Bootstrap sampling of contacts in a sparse Hi-C map.
+    """Bootstrap sampling of contacts in a sparse Hi-C map.
+
     Parameters
     ----------
     M : scipy.sparse.csr_matrix
@@ -1341,7 +1384,9 @@ def shortest_path_interpolation(matrix, alpha=1, strict=True):
     Also known as Boost-Hi-C (https://www.ncbi.nlm.nih.gov/pubmed/30615061)
     """
     matrix = np.array(matrix, np.float64)
-    contacts = distance_to_contact(to_distance(matrix, alpha=alpha), alpha=alpha)
+    contacts = distance_to_contact(
+        to_distance(matrix, alpha=alpha), alpha=alpha
+    )
     if not strict:
         return contacts
     else:
@@ -1440,7 +1485,8 @@ def distance_diagonal_law(matrix, positions=None, circular=False):
 
     intra_contacts = []
     inter_contacts = [
-        np.average(np.diagonal(matrix, j)) for j in range(max_intra_distance, n)
+        np.average(np.diagonal(matrix, j))
+        for j in range(max_intra_distance, n)
     ]
     for j in range(max_intra_distance):
         D = np.diagonal(matrix, j)
@@ -1472,9 +1518,13 @@ def rippe_parameters(matrix, positions, lengths=None, init=None, circ=False):
         for j in range(1, i):
             mean_length = (lengths[i] + lengths[j]) / 2.0
             if positions[i] < positions[j]:
-                d = ((positions[j] - positions[i] - lengths[i]) + mean_length) / 1000.0
+                d = (
+                    (positions[j] - positions[i] - lengths[i]) + mean_length
+                ) / 1000.0
             else:
-                d = ((positions[i] - positions[j] - lengths[j]) + mean_length) / 1000.0
+                d = (
+                    (positions[i] - positions[j] - lengths[j]) + mean_length
+                ) / 1000.0
 
             bins.append(np.abs(d))
             measurements.append(matrix[i, j])
@@ -1545,7 +1595,9 @@ def estimate_param_rippe(measurements, bins, init=None, circ=False):
                 0.53
                 * (param[0] ** -3.0)
                 * np.power((param[1] * x / param[0]), (param[2]))
-                * np.exp((d - 2) / ((np.power((param[1] * x / param[0]), 2) + d)))
+                * np.exp(
+                    (d - 2) / ((np.power((param[1] * x / param[0]), 2) + d))
+                )
             )
 
         return rippe
@@ -1622,7 +1674,10 @@ def null_model(
     elif model == "distance":
         distances = distance_diagonal_law(matrix, positions)
         N = np.array(
-            [[distances[min(abs(i - j), n)] for i in range(n)] for j in range(n)]
+            [
+                [distances[min(abs(i - j), n)] for i in range(n)]
+                for j in range(n)
+            ]
         )
 
     elif model == "rippe":
@@ -1684,10 +1739,18 @@ def null_model(
         return N
 
 
-def model_norm(matrix, positions=None, lengths=None, model="uniform", circ=False):
+def model_norm(
+    matrix, positions=None, lengths=None, model="uniform", circ=False
+):
 
     N = null_model(
-        matrix, positions, lengths, model, noisy=False, circ=circ, sparsity=True
+        matrix,
+        positions,
+        lengths,
+        model,
+        noisy=False,
+        circ=circ,
+        sparsity=True,
     )
     return matrix / shortest_path_interpolation(N, strict=True)
 
@@ -1700,7 +1763,9 @@ def trim_structure(struct, filtering="cube", n=2):
 
     if filtering == "sphere":
         R = (np.std(X) ** 2 + np.std(Y) ** 2 + np.std(Z) ** 2) * (n ** 2)
-        f = (X - np.mean(X)) ** 2 + (Y - np.mean(Y)) ** 2 + (Z - np.mean(Z)) ** 2 < R
+        f = (X - np.mean(X)) ** 2 + (Y - np.mean(Y)) ** 2 + (
+            Z - np.mean(Z)
+        ) ** 2 < R
 
     if filtering == "cube":
         R = min(np.std(X), np.std(Y), np.std(Z)) * n
@@ -1711,7 +1776,9 @@ def trim_structure(struct, filtering="cube", n=2):
     if filtering == "percentile":
         f = np.ones(len(X))
         for C in (X, Y, Z):
-            f *= np.abs(C - np.mean(C)) < np.percentile(np.abs(C - np.mean(C)), n)
+            f *= np.abs(C - np.mean(C)) < np.percentile(
+                np.abs(C - np.mean(C)), n
+            )
 
     return np.array([X[f], Y[f], Z[f]])
 
@@ -1772,7 +1839,9 @@ def scalogram(M, circ=False, max_range=False):
                 N[i, j] = M[i, i - j :].sum() + M[i, : i + j - n + 1].sum()
             elif circ and i < j and i + j >= n:
                 N[i, j] = (
-                    M[i, i - j :].sum() + M[i, :].sum() + M[i, : i + j - n + 1].sum()
+                    M[i, i - j :].sum()
+                    + M[i, :].sum()
+                    + M[i, : i + j - n + 1].sum()
                 )
     return N
 
@@ -1830,7 +1899,7 @@ def compartments(M, normalize=True):
     """
 
     n = M.shape[0]
-    if not type(M) is np.ndarray:
+    if type(M) is not np.ndarray:
         M = np.array(M)
 
     if M.shape[0] != M.shape[1]:
@@ -1923,7 +1992,7 @@ def compartments_sparse(M, normalize=True):
     N.data /= dist_vals[abs(N.row - N.col)]
     N = N.tocsr()
     # Make matrix symmetric (in case of upper triangle)
-    if (abs(N-N.T)>1e-10).nnz != 0:
+    if (abs(N - N.T) > 1e-10).nnz != 0:
         N = N + N.T
         N.setdiag(N.diagonal() / 2)
         N.eliminate_zeros()
